from uuid import uuid4

from orchestrator.forms import FormPage
from orchestrator.targets import Target
from orchestrator.types import FormGenerator, State
from orchestrator.types import SubscriptionLifecycle, UUIDstr
from orchestrator.workflow import done, init, step, workflow
from orchestrator.workflows.steps import resync, set_status
from orchestrator.workflows.steps import store_process_subscription
from orchestrator.workflows.utils import wrap_create_initial_input_form

from gso.products.product_blocks import site as site_pb
from gso.products.product_types import site


def initial_input_form_generator(product_name: str) -> FormGenerator:
    class CreateSiteForm(FormPage):
        class Config:
            title = product_name

        site_name: str
        site_city: str
        site_country: str
        site_country_code: str
        site_latitude: float
        site_longitude: float
        site_bgp_community_id: int
        site_internal_id: int
        site_tier: site_pb.SiteTier

    user_input = yield CreateSiteForm

    return user_input.dict()


@step("Create subscription")
def create_subscription(product: UUIDstr) -> State:
    subscription = site.SiteInactive.from_product_id(product, uuid4())

    return {
        "subscription": subscription,
        "subscription_id": subscription.subscription_id,
    }


@step("Initialize subscription")
def initialize_subscription(
    subscription: site.SiteInactive,
    site_name: str,
    site_city: str,
    site_country: str,
    site_country_code: str,
    site_latitude: float,
    site_longitude: float,
    site_bgp_community_id: int,
    site_internal_id: int,
    site_tier: site_pb.SiteTier,
) -> State:
    subscription.site.site_name = site_name
    subscription.site.site_city = site_city
    subscription.site.site_country = site_country
    subscription.site.site_country_code = site_country_code
    subscription.site.site_latitude = site_longitude
    subscription.site.site_longitude = site_latitude
    subscription.site.site_bgp_community_id = site_bgp_community_id
    subscription.site.site_internal_id = site_internal_id
    subscription.site.site_tier = site_tier

    subscription.description = f"Site {site_name}"

    subscription = site.SiteProvisioning.from_other_lifecycle(
        subscription, SubscriptionLifecycle.PROVISIONING
    )

    return {"subscription": subscription}


@workflow(
    "Create Site",
    initial_input_form=wrap_create_initial_input_form(
        initial_input_form_generator
    ),
    target=Target.CREATE,
)
def create_site():
    return (
        init
        >> create_subscription
        >> store_process_subscription(Target.CREATE)
        >> initialize_subscription
        >> set_status(SubscriptionLifecycle.ACTIVE)
        >> resync
        >> done
    )
